@@ -142,8 +142,8 @@ translate_message2(error(ISO, SWI)) -->
     swi_location(SWI),
     term_message(ISO),
     swi_extra(SWI).
-translate_message2('$aborted') -->
-    [ 'Execution Aborted' ].
+translate_message2(unwind(Term)) -->
+    unwind_message(Term).
 translate_message2(message_lines(Lines), L, T) :- % deal with old C-warning()
     make_message_lines(Lines, L, T).
 translate_message2(format(Fmt, Args)) -->
@@ -384,6 +384,11 @@ syntax_error(undefined_char_escape(C)) -->
     [ 'Unknown character escape in quoted atom or string: `\\~w\''-[C] ].
 syntax_error(void_not_allowed) -->
     [ 'Empty argument list "()"' ].
+syntax_error(Term) -->
+    { compound(Term),
+      compound_name_arguments(Term, Syntax, [Text])
+    }, !,
+    [ '~w expected, found '-[Syntax], ansi(code, '"~w"', [Text]) ].
 syntax_error(Message) -->
     [ '~w'-[Message] ].
 
@@ -637,6 +642,23 @@ thread_context -->
 thread_context -->
     [].
 
+		 /*******************************
+		 *        UNWIND MESSAGES	*
+		 *******************************/
+
+unwind_message(Var) -->
+    { var(Var) }, !,
+    [ 'Unknown unwind message: ~p'-[Var] ].
+unwind_message(abort) -->
+    [ 'Execution Aborted' ].
+unwind_message(halt(_)) -->
+    [].
+unwind_message(thread_exit(Term)) -->
+    [ 'Invalid thread_exit/1.  Payload: ~p'-[Term] ].
+unwind_message(Term) -->
+    [ 'Unknown "unwind" exception: ~p'-[Term] ].
+
+
                  /*******************************
                  *        NORMAL MESSAGES       *
                  *******************************/
@@ -764,6 +786,8 @@ prolog_message(unknown_in_module_user) -->
     ].
 prolog_message(untable(PI)) -->
     [ 'Reconsult: removed tabling for ~p'-[PI] ].
+prolog_message(unknown_option(Set, Opt)) -->
+    [ 'Unknown ~w option: ~p'-[Set, Opt] ].
 
 
                  /*******************************
@@ -1989,7 +2013,6 @@ default_theme(message(Level),         Attrs) :-
     prolog:message_prefix_hook/2.
 :- thread_local
     user:thread_message_hook/3.
-:- '$hide'((push_msg/1,pop_msg/0)).
 :- '$notransact'((user:message_hook/3,
                   prolog:message_prefix_hook/2,
                   user:thread_message_hook/3)).
@@ -2005,9 +2028,9 @@ print_message(Level, _Term) :-
     !.
 print_message(Level, Term) :-
     setup_call_cleanup(
-        push_msg(Term, Stack),
+        notrace(push_msg(Term, Stack)),
         ignore(print_message_guarded(Level, Term)),
-        pop_msg(Stack)),
+        notrace(pop_msg(Stack))),
     !.
 print_message(Level, Term) :-
     (   Level \== silent
