@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2020, University of Amsterdam
+    Copyright (c)  2002-2023, University of Amsterdam
                               VU University Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -36,6 +37,7 @@
 :- module(html_write,
           [ reply_html_page/2,          % :Head, :Body
             reply_html_page/3,          % +Style, :Head, :Body
+            reply_html_partial/1,       % +HTML
 
                                         % Basic output routines
             page//1,                    % :Content
@@ -73,7 +75,7 @@
           ]).
 :- use_module(html_quasiquotations, [html/4]).
 :- autoload(library(apply),[maplist/3,maplist/4]).
-:- autoload(library(debug),[debug/3]).
+:- use_module(library(debug),[debug/3]).
 :- autoload(library(error),
 	    [must_be/2,domain_error/2,instantiation_error/1]).
 :- autoload(library(lists),
@@ -834,7 +836,7 @@ html_quoted(Text) -->
 %   Quote the value  according  to   the  rules  for  tag-attributes
 %   included in double-quotes.  Note   that  -like  html_quoted//1-,
 %   attributed   values   printed   through   html//1   are   quoted
-%   atomatically.
+%   automatically.
 %
 %   @tbd    Assumes UTF-8 encoding of the output.
 
@@ -906,7 +908,7 @@ html_receive(Id) -->
 %!  html_receive(+Id, :Handler)// is det.
 %
 %   This extended version of html_receive//1   causes  Handler to be
-%   called to process all messages posted to the channal at the time
+%   called to process all messages posted to the channel at the time
 %   output  is  generated.  Handler  is    called  as  below,  where
 %   `PostedTerms` is a list of  Module:Term   created  from calls to
 %   html_post//2. Module is the context module of html_post and Term
@@ -936,7 +938,7 @@ html_noreceive(Id) -->
 %   Collect  posted  tokens  and  copy    them  into  the  receiving
 %   mailboxes. Mailboxes may produce output for  each other, but not
 %   cyclic. The current scheme to resolve   this is rather naive: It
-%   simply permutates the mailbox resolution order  until it found a
+%   simply  permutes  the mailbox resolution order  until it found a
 %   working one. Before that, it puts   =head= and =script= boxes at
 %   the end.
 
@@ -1348,9 +1350,12 @@ html_print_length([H|T], L0, L) :-
 %!  reply_html_page(:Head, :Body) is det.
 %!  reply_html_page(+Style, :Head, :Body) is det.
 %
-%   Provide the complete reply as required  by http_wrapper.pl for a
-%   page constructed from Head and   Body. The HTTP =|Content-type|=
-%   is provided by html_current_option/1.
+%   Provide the complete reply as required by http_wrapper.pl for a page
+%   constructed  from  Head  and  Body.  The  HTTP  =|Content-type|=  is
+%   provided by html_current_option/1.
+%
+%   @see  reply_html_partial/1  to  avoid  adding   a  ``DOCTYPE``,  and
+%   required outer HTML elements such as ``<html>``.
 
 reply_html_page(Head, Body) :-
     reply_html_page(default, Head, Body).
@@ -1362,6 +1367,24 @@ reply_html_page(Style, Head, Body) :-
     print_html(HTML).
 
 
+%!  reply_html_partial(+HTML) is det.
+%
+%   Reply with partial HTML  document.  The   reply  only  contains  the
+%   element from HTML, i.e., this predicate   does not add a ``DOCTYPE``
+%   header, ``<html>``, ``<head>`` or  ``<body>``.   It  is intended for
+%   JavaScript handlers that request a partial  document and insert that
+%   somewhere into the existing page DOM.
+%
+%   @see reply_html_page/3 to reply with a complete (valid) HTML page.
+%   @since 9.1.20
+
+reply_html_partial(HTML) :-
+    html_current_option(content_type(Type)),
+    phrase(html(HTML), Tokens),
+    format('Content-type: ~w~n~n', [Type]),
+    print_html(Tokens).
+
+
 %!  html_header_hook(+Style) is nondet.
 %
 %   This multifile hook  is  called   just  before  the ``Content-type``
