@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2024, University of Amsterdam
+    Copyright (c)  2011-2025, University of Amsterdam
 			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -47,6 +47,7 @@
 #include "pl-ctype.h"
 #include "pl-funct.h"
 #include "../pl-arith.h"
+#include "../pl-index.h"
 #include "../pl-tabling.h"
 #include "../pl-fli.h"
 #include "../pl-write.h"
@@ -68,7 +69,6 @@
 #include <dlfcn.h>
 #endif
 
-#undef bool
 #define ison(s, a)         ((s)->flags & (a))
 #define isoff(s, a)        (!ison((s), (a)))
 
@@ -488,7 +488,7 @@ setRationalSyntax(atom_t a, unsigned int *flagp)
 
 
 
-static int
+static bool
 setUnknown(term_t value, atom_t a, Module m)
 { unsigned int flags = m->flags & ~(UNKNOWN_MASK);
 
@@ -524,6 +524,22 @@ setUnknown(term_t value, atom_t a, Module m)
 }
 
 
+static bool
+setUnknownOption(term_t value, atom_t a)
+{ GET_LD
+
+  if ( a == ATOM_ignore )
+  { LD->prolog_flag.unknown_option = OPT_UNKNOWN_IGNORE;
+  } else if ( a == ATOM_warning )
+  { LD->prolog_flag.unknown_option = OPT_UNKNOWN_WARNING;
+  } else if ( a == ATOM_error )
+  { LD->prolog_flag.unknown_option = OPT_UNKNOWN_ERROR;
+  } else
+    return PL_domain_error("unknown_option", value);
+
+  return true;
+}
+
 static int
 checkOnError(term_t value, atom_t a, atom_t key)
 { if ( a == ATOM_print || a == ATOM_halt || a == ATOM_status )
@@ -915,6 +931,8 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, unsigned shor
       return tbl_set_restraint_flag(value, k) ? PSEUDO_FLAG : NULL;
     if ( is_arith_flag(k) )
       return set_arith_flag(value, k) ? PSEUDO_FLAG : NULL;
+    if ( ci_is_flag(k) )
+      return ci_set_flag(value, k) ? PSEUDO_FLAG : NULL;
 
 #ifdef O_PLMT
     if ( GD->statistics.threads_created > 1 )
@@ -1124,6 +1142,8 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, unsigned shor
       { rval = setRationalSyntax(a, &m->flags);
       } else if ( k == ATOM_unknown )
       { rval = setUnknown(value, a, m);
+      } else if ( k == ATOM_unknown_option )
+      { rval = setUnknownOption(value, a);
       } else if ( k == ATOM_on_error || k == ATOM_on_warning )
       { rval = checkOnError(value, a, k);
       } else if ( k == ATOM_write_attributes )
@@ -1203,7 +1223,7 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, unsigned shor
       { if ( i < 0 )
 	  return PL_error(NULL, 0, NULL, ERR_DOMAIN,
 			  ATOM_not_less_than_zero, value),NULL;
-	LD->yield.frequency = i/16;
+	LD->yield.frequency = (i+15)/16;
       }
 
       f->value.i = i;
@@ -1487,6 +1507,8 @@ unify_prolog_flag_value(DECL_LD Module m, atom_t key, prolog_flag *f, term_t val
   { return tbl_get_restraint_flag(val, key) == true;
   } else if ( is_arith_flag(key) )
   { return get_arith_flag(val, key) == true;
+  } else if ( ci_is_flag(key) )
+  { return ci_get_flag(val, key);
   }
 
   switch(f->flags & FT_MASK)
@@ -1570,7 +1592,10 @@ typedef struct
   Module module;
 } prolog_flag_enum;
 
-foreign_t
+#define pl_prolog_flag5(key, value, scope, access, type, h) \
+	LDFUNC(pl_prolog_flag5, key, value, scope, access, type, h)
+
+static foreign_t
 pl_prolog_flag5(DECL_LD term_t key, term_t value,
 		term_t scope, term_t access, term_t type,
 		control_t h)
@@ -1695,6 +1720,32 @@ pl_prolog_flag5(DECL_LD term_t key, term_t value,
   fail;
 }
 
+bool
+PL_get_prolog_flag(atom_t name, term_t value)
+{ GET_LD
+  fid_t fid;
+  term_t key;
+  bool rc;
+
+  if ( (fid=PL_open_foreign_frame()) &&
+       (key=PL_new_term_ref()) &&
+       PL_put_atom(key, name) )
+  { struct foreign_context ctx = {
+      .context = 0,
+      .control = FRG_FIRST_CALL,
+      .engine  = LD
+    };
+
+    rc = pl_prolog_flag5(key, value, 0, 0, 0, &ctx);
+  } else
+    rc = false;
+
+  if ( fid )
+    PL_close_foreign_frame(fid);
+
+  return rc;
+}
+
 static
 PRED_IMPL("$current_prolog_flag", 5, dcurrent_prolog_flag, PL_FA_NONDETERMINISTIC)
 { PRED_LD
@@ -1874,6 +1925,7 @@ initPrologFlags(void)
 #endif
 #ifdef O_PLMT
   setPrologFlag("threads",	FT_BOOL, !GD->options.nothreads, 0);
+  setPrologFlag("engines",	FT_BOOL, true, 0);
   if ( GD->options.xpce >= 0 )
     setPrologFlag("xpce",	FT_BOOL, GD->options.xpce, 0);
   setPrologFlag("system_thread_id", FT_INTEGER|FF_READONLY, (intptr_t)0);
@@ -1883,6 +1935,9 @@ initPrologFlags(void)
 #else
   setPrologFlag("threads",	FT_BOOL|FF_READONLY, false, 0);
   setPrologFlag("gc_thread",    FT_BOOL|FF_READONLY, false, PLFLAG_GCTHREAD);
+#ifdef O_ENGINES
+  setPrologFlag("engines",	FT_BOOL, true, 0);
+#endif
 #endif
 #ifdef O_DDE
   setPrologFlag("dde", FT_BOOL|FF_READONLY, true, 0);
@@ -1952,6 +2007,8 @@ initPrologFlags(void)
   setPrologFlag("portable_vmi", FT_BOOL, true, PLFLAG_PORTABLE_VMI);
   setPrologFlag("traditional", FT_BOOL|FF_READONLY, GD->options.traditional, 0);
   setPrologFlag("unknown", FT_ATOM, "error");
+  setPrologFlag("unknown_option", FT_ATOM, "ignore");
+  LD->prolog_flag.unknown_option = OPT_UNKNOWN_IGNORE;
   setPrologFlag("debug", FT_BOOL, false, 0);
   setPrologFlag("debug_on_interrupt", FT_BOOL,
 		truePrologFlag(PLFLAG_DEBUG_ON_INTERRUPT),
@@ -2019,6 +2076,7 @@ initPrologFlags(void)
 		truePrologFlag(PLFLAG_SIGNALS), PLFLAG_SIGNALS);
   setPrologFlag("packs", FT_BOOL, GD->cmdline.packs, 0);
   setPrologFlag("heartbeat", FT_INTEGER, (intptr_t)0);
+  setPrologFlag("halt_grace_time", FT_FLOAT, (double)1.0);
 
 #if defined(__WINDOWS__) && defined(_DEBUG)
   setPrologFlag("kernel_compile_mode", FT_ATOM|FF_READONLY, "debug");
