@@ -2309,8 +2309,10 @@ clear_frame_vars(LocalFrame target)
 	if ( ch->type == CHP_JUMP )
 	{ PC = ch->value.pc;
 	  DEBUG(0,
-		{ Code codes = fr->clause->value.clause->codes;
-		  assert(PC < &codes[codes[-1]]);
+		{ if ( isoff(fr->predicate, P_FOREIGN) )
+		  { Code codes = fr->clause->value.clause->codes;
+		    assert(PC < &codes[codes[-1]]);
+		  }
 		});
 	} else
 	  PC = NULL;
@@ -2627,12 +2629,8 @@ PRED_IMPL("prolog_choice_attribute", 3, prolog_choice_attribute, 0)
       return PL_unify_int64(A3, offset);
     return false;
   } else if ( key == ATOM_clause )
-  { if ( ch->type == CHP_CLAUSE )
-    { return PL_unify_clref(A3, ch->value.clause.cref->value.clause);
-    } else if ( choice_type_atom(ch) == ATOM_clause )
-    { Sdprintf("S_ALLCLAUSES not yet supported\n");
-    }
-    return false;
+  { assert(ch->type == CHP_CLAUSE);
+    return PL_unify_clref(A3, ch->value.clause.cref->value.clause);
   } else
     return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_key, A2);
 
