@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2022-20244, SWI-Prolog Solutions b.v.
+    Copyright (c)  2022-2025, SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -49,47 +49,43 @@ user:file_search_path(scasp,  Dir) :-
     getenv('SCASP_HOME', Dir).
 
 :- http_handler('/', http_redirect(see_other, '/wasm/'), []).
-:- http_handler('/wasm/shell',
-                http_reply_file(
-                    web('shell.html'),
-                    [ headers([ 'Cross-Origin-Opener-Policy'('same-origin'),
-                                'Cross-Origin-Embedder-Policy'('require-corp')
-                              ])]),
-                []).
-:- http_handler('/wasm/test',
-                http_reply_file(
-                    web('test.html'),
-                    [ headers([ 'Cross-Origin-Opener-Policy'('same-origin'),
-                                'Cross-Origin-Embedder-Policy'('require-corp')
-                              ])]),
-                []).
-:- http_handler('/wasm/cbg',
-                http_reply_file(
-                    web('cbg.html'),
-                    [ headers([ 'Cross-Origin-Opener-Policy'('same-origin'),
-                                'Cross-Origin-Embedder-Policy'('require-corp')
-                              ])]),
-                []).
+:- http_handler('/wasm/shell', reply_html_test('shell.html'), []).
+:- http_handler('/wasm/test',  reply_html_test('test.html'), []).
+:- http_handler('/wasm/cbg',   reply_html_test('cbg.html'), []).
 :- http_handler('/wasm/',
                 http_reply_from_files(web(.), [static_gzip(true)]), [prefix]).
 
 
+reply_html_test(File, Request) :-
+    http_reply_file(web(File),
+                    [ headers([ 'Cross-Origin-Opener-Policy'('same-origin'),
+                                'Cross-Origin-Embedder-Policy'('require-corp')
+                              ])],
+                    Request).
+
 :- if(absolute_file_name(scasp(.), _, [file_type(directory), file_errors(fail)])).
 :- http_handler('/wasm/scasp/', http_reply_from_files(scasp(.), []), [prefix]).
 :- endif.
 
 
-:- initialization(server_loop, main).
+:- initialization(main, main).
+
+opt_type(port,        port,        nonneg).
+opt_type(p,           port,        nonneg).
+opt_type(interactive, interactive, boolean).
+opt_type(i,           i,           boolean).
 
-opt_type(port, port, nonneg).
 opt_help(port, "Port to listen to (default 8080)").
+opt_help(interactive, "Become interactive").
 
-server :-
-    current_prolog_flag(argv, Argv),
-    argv_options(Argv, _Positonal, Options),
+server(Options) :-
     merge_options(Options, [port(8080)], Options1),
     http_server(Options1).
 
-server_loop :-
-    server,
-    thread_get_message(quit).
+main(Argv) :-
+    argv_options(Argv, _Pos, Options),
+    server(Options),
+    (   option(interactive(true), Options)
+    ->  cli_enable_development_system
+    ;   thread_get_message(quit)
+    ).
